/////////////////////////////////////////////////////////////////////////////
// Name:        AutoCounterTiler.cpp
// Purpose:     Generate TAM tilesets which self-assemble into counters
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

// AutoCounterTiler.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "AutoCounterTiler.h"


void Usage()
{
  printf("\nUsage:  AutoCounterTiler.exe -base 2 -dir N -face E -out <tileset output file> [-seed <seed output file>] [-tilesim]");
  printf("\n        AutoCounterTiler.exe -fw 5 -base 3 -dir E -face N -out <tileset output file> [-seed <seed output file>] [-tilesim]");
  printf("\n");
  printf("\n  -fw      : generate a fixed-width counter of the specified width (if");
  printf("\n             this option is omitted, the counter will log-width)");
  printf("\n  -base    : the base number of the counter");
  printf("\n  -dir     : the growth direction of the counter (N, S, E, or W)");
  printf("\n  -face    : the direction that the counter 'faces' (N, S, E, or W)");
  printf("\n  -out     : name of the output file for the generated tileset");
  printf("\n  -seed    : [optional] name of the output file for the generated tile seed");
  printf("\n  -tilesim : [optional] if this option is present, the seed file will be");
  printf("\n             generated in tilesim format, otherwise trend format");
  printf("\n");
  printf("\n  Note:  Defaults to a log-width binary counter that grows North and faces East");
  printf("\n");
  exit(0);
}

int main(int argc, char* argv[])
{
  int iFixedWidth = 0;
  int iBase = 2;
  int iDirGrow = NORTH;
  int iDirFace = EAST;
  std::string TileOutFile;
  std::string SeedOutFile;
  bool bTrendSeed = true;
  bool bFaceSpecd = false;

  for (int i=1; i<argc; i++) {
    char* foo = argv[i];
    if (strcmp(argv[i], "-fw") == 0) {
      if (argc >= i+1) {
        long lfw = atol(argv[++i]);
        iFixedWidth = (int)lfw;
      } else {
        Usage();
      }
    } else if (strcmp(argv[i], "-base") == 0) {
      if (argc >= i+1) {
        long lbase = atol(argv[++i]);
        iBase = (int)lbase;
      } else {
        Usage();
      }
    } else if (strcmp(argv[i], "-dir") == 0) {
      if (argc >= i+1) {
        iDirGrow = DirectionIntFromString(argv[++i]);
      } else {
        Usage();
      }
    } else if (strcmp(argv[i], "-face") == 0) {
      if (argc >= i+1) {
        iDirFace = DirectionIntFromString(argv[++i]);
        bFaceSpecd = true;
      } else {
        Usage();
      }
    } else if (strcmp(argv[i], "-out") == 0) {
      if (argc >= i+1) {
        TileOutFile = argv[++i];
      } else {
        Usage();
      }
    } else if (strcmp(argv[i], "-seed") == 0) {
      if (argc >= i+1) {
        SeedOutFile = argv[++i];
      } else {
        Usage();
      }
    } else if (strcmp(argv[i], "-tilesim") == 0) {
      bTrendSeed = false;
    }
  }
  if (TileOutFile.length() <= 0) {
    Usage();
  }

  if (!bFaceSpecd) {
    iDirFace = (iDirGrow + 1) % 4;
  }

  if ((iDirFace == iDirGrow) || (iDirFace == ((iDirGrow + 2) % 4))) {
    printf("\n\n\nERROR:  Invalid direction specified for 'face' argument.\n\n");
    Usage();
  }

  //////////////////////////////////////////////////////////////////////
  //  Create the tile set
  //    Initially they will be built assuming growth north and facing east,
  //    then rotated/reflected as necessary

  if (iFixedWidth == 0) {
    printf("Generating a log-width base %d counter that grows %s and faces %s...\n", iBase, DirectionStringFromInt(iDirGrow).c_str(), DirectionStringFromInt(iDirFace).c_str());
  } else {
    printf("Generating a fixed-width base %d counter of width %d that grows %s and faces %s...\n", iBase, iFixedWidth, DirectionStringFromInt(iDirGrow).c_str(), DirectionStringFromInt(iDirFace).c_str());
  }

  TileSet ts;
  if (!GenerateCounterTileSet(iBase, iDirGrow, iDirFace, ((iFixedWidth == 0) ? COUNTER_LOGWIDTH : COUNTER_FIXEDWIDTH), &ts)) {
    printf("\nError:  Tileset generation failed.  Aborting.\n");
    exit(1);
  }

  //////////////////////////////////////////////////////////////////////
  //  Output the tileset

  std::ofstream out(TileOutFile.c_str());
  if (!out.is_open()) {
    printf("Error:  Failed to open the output file '%s'\n", TileOutFile.c_str());
    exit(1);
  }

  printf("\nOutputting tileset to file: '%s'...\n\n", TileOutFile.c_str());
  for (size_t j=0; j<ts.size(); j++) {
    ts[j].Print(&out);
    out << "\n";
  }
  out.close();

  //////////////////////////////////////////////////////////////////////
  //  Output the seed definition

  printf("Outputting seed definition to file: '%s'...\n\n", SeedOutFile.c_str());

  std::ofstream seedOut(SeedOutFile.c_str());
  if (!seedOut.is_open()) {
    printf("Error:  Failed to open the seed output file '%s'\n", SeedOutFile);
    exit(1);
  }

  std::string tdpFile = TileOutFile.substr((TileOutFile.find_last_of('\\')+1), TileOutFile.length());
  seedOut << tdpFile;
  seedOut << "\n";
  if (iFixedWidth == 0) {
    seedOut << "SEED ";
    if (bTrendSeed) {
      seedOut << "35 35\n";
    } else {
      seedOut << "35 35\n";
    }
  } else {
    seedOut << "SEEDL ";
    if (bTrendSeed) {
      seedOut << "35 35\n";
    } else {
      seedOut << "35 35\n";
    }

    int mx = 35;
    int my = 35;

    for (int i=1; i<=iFixedWidth-2; i++) {
      int x = 35;
      int y = 35;

      if ((iDirGrow == NORTH) || (iDirGrow == SOUTH)) {
        if (iDirFace == EAST) {
          x = x - i;
          mx = 35 - iFixedWidth + 1;
        } else {
          x = x + i;
          mx = 35 + iFixedWidth - 1;
        }
      } else if ((iDirGrow == EAST) || (iDirGrow == WEST)) {
        if (iDirFace == NORTH) {
          y = y - i;
          my = 35 - iFixedWidth + 1;
        } else {
          y = y + i;
          my = 35 + iFixedWidth - 1;
        }
      }

      seedOut << "SEEDINT ";
      if (bTrendSeed) {
        seedOut << x;
        seedOut << " ";
        seedOut << y;
        seedOut << "\n";
      } else {
        seedOut << y;
        seedOut << " ";
        seedOut << x;
        seedOut << "\n";
      }
    }

    //  Account for the fact that with width 2 there won't be any SEEDINT tiles
    if (iFixedWidth == 2) {
      if ((iDirGrow == NORTH) || (iDirGrow == SOUTH)) {
        if (iDirFace == EAST) {
          mx--;
        } else {
          mx++;
        }
      } else if ((iDirGrow == EAST) || (iDirGrow == WEST)) {
        if (iDirFace == NORTH) {
          my--;
        } else {
          my++;
        }
      }
    }

    seedOut << "SEEDM ";
    if (bTrendSeed) {
      seedOut << mx;
      seedOut << " ";
      seedOut << my;
      seedOut << "\n";
    } else {
      seedOut << my;
      seedOut << " ";
      seedOut << mx;
      seedOut << "\n";
    }
  }

  seedOut.close();

  printf("AutoCounterTiler has completed successfully.\n");
	return 0;
}

